;KitchenTimer100725

;Coding from the ATMega8L timer was used in order to have a "time left" before beep readout
;  This uses the 3.58 MHz xtl instead of 32,768 Hz crystal.
;  Too, 3 volts on the chip with 5.5 volts on the Sonalert is used here.

; August 12,2010
; Addition of "Repeat of Last Set Time"

;KitchenTimer100725.asm
;
; Divide by 8 fuse set

; Fuse set for Xtal 3.579545, 65 ms
;   3-8 Mz, 14CK, 65 ms, CKSEL=1101, SUT 11
;   (slowly rising power)

; With divide by 8 for final frequency

;Upgrade of Kitchen Timer in that no on/off switch necessary.
;   The 5 volt step-up circuit is powered down after each timing cycle.

;Timer starts at first button push
;Up to 10 seconds in between button pushes is allowed

;Current draw at idle = approx = 
;Short Dit Touch = one minute addition
;Longer Dah Touch = ten minute addition
;HoldDown Count = one hour addition

;To enter "commmand" mode, short dit count followed by 
;   Hold Down count (HC) of one  (Sonalert beeps for each HC)

;ClockCntDownInt sets r18, bit 0(1) every minute
;r24 is seconds counter - incremented one each second

;The low byte for the 1 second interrupt has a default of 79.  This can
;  be changed to any other number by the dit dah, dit dah, dit dah, 
;  dit dah entry combination (4 pairs of dit dah's).  Five beeps are 
;  heard. You can then make a hold down count of one or more beeps
;  to exit, ... or enter 8 dit dah combinations for a new default 
;  number other than 132.  Such entries are put into EEPROM and 
;  remembered permanently at power ups.  This allows a way to adjust for time interval for individual
;  chip variations.
;  
;  To make time check, use the dit dah, dit dah, dah, dah, dah, dah to
;	enter toggle on and off of beep every minute.  Three beeps are 
;	heard when entering this routine.



; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

.include "tn2313def.inc"


rjmp	START1
rjmp	START1
rjmp	START1
rjmp	START1
rjmp	ClockCntDwnInt	;one second interrupt for timing - counter A

	
START1:				;set up stack pointer and initialize cold start

ldi		r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out		SPL,R16				; Initialize stack pointer to end of internal RAM

ldi		r16,7		;Set bits 0-3 for output
out		DDRA,r16	;Execute data direction with PA0-3 as output

clr		r16
out		PortA,r16	;turn off ports

ser		r16			;Set to all ones for out port direction as output
out		DDRB,r16	;Set PORTB to output

clr		r16
out		PortB,r16	;ALL off when port pin is low

ldi		R16,251		;Set bit 2 (4), as input ... remaining bits as output
out		DDRD,R16	;execute data direction with bit 0 as input

clr		r16
out		PORTD,R16	;execute to set out ports low, and in port with
					; no input resistor

clr		r19			;clear flags register
clr		r23			;universal delay counter
clr		r24			;clear seconds counter
clr		r25			;clear 10 second wait counter
clr		r26			;lo byte of HIGH Timer Counter pair
clr		r27			;hi byte of HIGH Timer Counter pair
clr		r30			;low byte of LOW timer pair
clr		r31			;hi byte of LOW time pair

in		r16,TIMSK	;enable 16-bit match to A register
ori		r16,64		
out		TIMSK,r16

in		r16,TCCR1B
ori		r16,11		; divide by 64 on prescaler
out		TCCR1B,r16	;enable reset of counter to zero (bit3) after match detection.
					;and bits 0,1,2 select prescaling source as 64 of clock
					
					;3,579,545 Xtl with 8 division = 447,443
					;  divided by 64 = 6991.3 
ldi		r16,27		;comparator high byte for this crystal
out		OCR1AH,r16
ldi		r16,79		;default comparator low byte for crystal
out		OCR1AL,r16	;for one second intervals 
               
;bset	7			;enable global interrupts

clr		r19				;clear R19 and all r19 flags
sts		$006B,r19		;clear  seconds counter

rjmp	MainStart


;INTERRUPTS remain off until needed

;--------------------------------------------------------

;r19,5(32) = grabbed second match flag
;r19,6(64) = minute flag
;r19,7(128) = second flag

;r20 = elapsed seconds count
;r21 = grab current second store
;r22 = elapsed minute count
;r23 = number of beeps count

;r30,31 = total delay entered by push button


;---------------------------------------------------------


;%%%%%%  This section turns on power for period of six seconds to await entry of desired time interval
;  IF no entry is made, unit powers back down.  This section NOT used for KithenTimer100725


MainStart:

sbi		PortB,2			;ON sustain voltage line
rjmp	DecodeButtonPush


;-------------------------------

ButtonStall:				;Stalls when button is up.  It waits for button down

sbis	PIND,2				;skip if switch is up
rjmp	ButtonPush
rjmp	ButtonStall

;-------------------------------


;SUBROUTINE

;Button MUST be DOWN when entering this routine

ButtonPush:		;Takes in dit, dah, holddown, and coded input

push	r0		;number of code digits
push	r1		;low code byte
push	r2		;next high code byte
push	r3		;next high code byte
push	r4		;highest code byte
push	r18		;dit count
push	r20		;holddown count
push	r23		;universal counter
push	r24		;combined count
push	r28		;dah count
push	r30		;has total count of entry in Z register, r30,31
push	r31

;r30,r31 (Z register pair contain total value of entry)

clr		r18		;Dit Count
clr		r20		;HoldDown count
clr		r23		;universal counter
clr		r28		;dah count
clr		r0		;clear number of bits in code
clr		r1		;clear lowest code count
clr		r2		;clear next code count
clr		r3		;clear next code count
clr		r4		;clear next code count (clear all code counts)


				;r30,31 has total count

B1:

inc		r23			;universal counter
rcall	Delay40th
cpi		r23,60		;delay before first holddown count - 40=one second 
brsh	B3			;branch if - it is a holddown count

sbis	PIND,2		;skip if switch is NOT down
rjmp	B1


B2:

cpi		r23,20		;compare r23 for 20 (1/2 second dah count)
brlo	B2b			;branch if lower (dit count)


B2a:				;it is dah count

inc		r28			;increment dah count

sec					;set carry flag -- it is dah entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortB,0		;on Sonalert
rcall	Delay40th
rcall	Delay40th	;delay for longer on of sonalert
rcall	Delay40th
rcall	Delay40th
cbi		PortB,0		;off Sonalert
clr		r23
inc		r0			;increment number of code digits


B2d:				;no touch/switch up detection

sbis	PIND,2		;skip if switch is up
rjmp	B2e			;jump back - there is a new entry coming in

inc		r23
rcall	Delay40th
cpi		r23,70		;has switch been up 70 counts? 40=one second (1.75 seconds)
brsh	B4Decode1
rjmp	B2d


BPEnd1:

rjmp	BPEnd


B2e:

clr		r23
rjmp	B1


B2b:				;Dit count detected

inc		r18			;increment dit count

clc					;clear carry flag -- it is dit entry
rol		r1
rol		r2			;rotate data into registers r1-r4
rol		r3
rol		r4

sbi		PortB,0		;On Sonalert
rcall	Delay40th
cbi		PortB,0		;Off Sonalert

inc		r0			;increment number of code digits (r0)
clr		r23			;clear r23
rjmp	B2d


B3:					;Holddown count has been detected

inc		r20			;increment holddown count
sbi		PortB,0		;On Sonalert
rcall	Delay40th
rcall	Delay40th
cbi		PortB,0		;Off Sonalert for short beep

clr		r23			;reset r23 to zero count


B4:

inc		r23
rcall	Delay40th
sbic	PIND,2		;skip if switch is down
rjmp	B4a			;jump out and look for next entry

cpi		r23,30		;pause between continuous hold down of switch
brlo	B4

rjmp	B3


B4a:

clr		r23
rjmp	B2d			;Check for exit or new entry



B4Decode1:

;This section finalizes.  It combines counts

clr		r30
clr		r31			;ready registers for input

mov		r16,r28		;move dah count in r28 into r16


B4comb1:

tst		r16
breq	B4comb2

adiw 	r30,10		;add value of 10 in r30,31
dec		r16			
rjmp	B4comb1


B4comb2:

add		r30,r18		;add value in r18 (dit count) to r24
brcs	B4comb2a
rjmp	Bholdown1


B4comb2a:

inc		r31			;carry was set so increment higher byte


Bholdown1:

;Combined value now in Z register pair, r30,31

mov		r16,r20		;put hold down count into r16

Bholdown1b:

tst		r16			;test r20(r16), hold down count
breq	BPEnd
adiw	r30,50		;add fifty count to total
dec		r16
rjmp	Bholdown1b

;Complete combined value now in Z register pair


BPEnd:

;Store away in SRAM $00E7-00EB, Dit,Dah,HolddownCount,LSD Total, MSD Total

sts		$0070,r0		;number of code digits
sts		$0071,r1		;low byte of code digit
sts		$0072,r2		;next byte of code digit
sts		$0073,r3		;high byte of code digit

sts		$0074,r18		;number of dits
sts		$0075,r28		;number of dahs
sts		$0076,r20		;number of hold down counts
sts		$0077,r30		;LSD of total count
sts		$0078,r31		;MSD of total count

pop		r31
pop		r30
pop		r28
pop		r24
pop		r23
pop		r20
pop		r18
pop		r4
pop		r3
pop		r2
pop		r1
pop		r0

ret


;-------------------------------

DecodeButtonPush:

rcall	ButtonPush

lds		r16,$0070	;No of digits in code string

cpi		r16,6		;is this valid code? r16 with 6 digits?

breq	DBPcode
rjmp	Main2


DBPcode:

lds		r16,$0071	;load low byte of code digit


DBPcode1:

cpi		r16,31		;. _ _ _ _ _  dit,dah,dah,dah,dah,dah (31) (Run FAVORITE EEPROM time)
brne	DBPcode2
rjmp	RunEEPROMTime


DBPcode2:

cpi		r16,15		;. . _ _ _ _  dit,dit,dah,dah,dah,dah (15) Up Counter readout every minute
brne	DBPcode3
rjmp	DBPcodeA	;Start Up Counter - one minute increments


DBPcode3:

cpi		r16,7		;. . . _ _ _  dit,dit,dit,dah,dah,dah (15) - Beep at set Intervals
brne	DBPcode4
rjmp	BeepIntervalLoop	;run beeping at regular intervals (set by Program Interval in EEPROM)


DBPcode4:

cpi		r16,3		;. . . . _ _  dit,dit,dit,dit,dah,dah (7)
brne	DBPcode5		;continue decode cycle
rcall	ProgramEEPROM	;Program EEPROM (FAVORITE) time interval
rjmp	PowerOff


DBPcode5:

cpi		r16,1		;. . . . . _  dit,dit,dit,dit,dit,dah (1) Program beep interval in minutes
brne	DBPcode6
rcall	ProgramBeepInterval	; Beep at interval of minutes entered here
rjmp	PowerOff


DBPcode6:

cpi		r16,5		; . . . - . -  dit,dit,dit,dah,dit,dah (5) Program beep interval in seconds
brne	DBPcode7
rcall	ProgramBeepInterval2	;Beep at interval of seconds entered here
rjmp	PowerOff


DBPcode7:

cpi		r16,4		; . . . - . .  dit,dit,dit,dah,dit,dit (4) Metronome Program
brne	DBPend
rcall	Metronome
rjmp	PowerOff


DBPEnd:

rjmp	Main2


;---

DBPcodeA:			;COUNT UP TIMER

clr		r28
clr		r29			;clear r28,29 Y register pair

ldi		r16,1		;start seconds one second in
sts		$006B,r16	;grab present second value

sei					;ON interrupts


DBPcodeB:			;Beeps out elapsed time every minute.  3 short beeps beep out hours, 2 short beeps = 10 minute
					;  intervals,  1 short beep = minutes


lds		r16,$006B	;grab current second value

tst		r16
brne	DBPcodeC

adiw	r28,1		;increment r28,29 pair by one minute

rcall	BeepOutElapsed	;beep out current elapsed time - values in r28,29

rjmp	DBPcodeB	;loop back for test of new second


DBPcodeC:

sbis	PIND,2		;skip if switch is up
rjmp	PowerOff

rjmp	DBPcodeB


;-------------------------

RunPreviousTime:

ldi		r16,2
rcall	VariableBeep	;beep sonalert two times

lds		r30,$61
lds		r31,$62		;low pair bytes

lds		r26,$63
lds		r27,$64		;high pair bytes

; RELOADED past information for delay cycle

rjmp	Main2b		;jump back to repeat the last timing cycle.


;-------------------------

RunEEPROMTime:

ldi		r16,3		
rcall	VariableBeep	;three beeps

rcall	UseEEPROM	;Get stored data in EEPROM for time and put into r30,31, r26,27
rjmp	Main2b		;use EEPROM data for timing


;-------------------------

ProgramEEPROM:			;NOT subroutine - programs time into EEPROM

ldi		r16,3
rcall	VariableBeep		;beep sonalert 3 times


EEcode1:

sbic	PIND,2		;skip if button is down (WAIT)
rjmp	EEcode1

rcall	ButtonPush	;allow number of minutes to be entered
rcall	CalculateTotalSeconds

; r30,31 pair have low byte data;   r26,27 pair have high byte data

ldi		r17,$00		;start location for EEPROM data
mov		r16,r30		;move low byte of low data pair into $00
rcall	WriteEEPROM

inc		r17			;to next data location
mov		r16,r31		;move high byte of low data pair into $01
rcall	WriteEEPROM

inc		r17			;to next data location
mov		r16,r26		;move low byte of high data pair into $02
rcall	WriteEEPROM

inc		r17			;to next data location
mov		r16,r27		;move high byte of high data pair into $03
rcall	WriteEEPROM

ldi		r16,3
rcall	VariableBeep	;beep sonalert 3 times

rjmp 	PowerOff


;-------------------------

;  NOT A SUBROUTINE!!!

DecodeButtonPush2:		;Used for exit of a completed timing cycle to (1)exit,
						;  (2)repeat a timing cycle, or use (3)EEPROM time cycle.

rcall	ButtonPush

lds		r16,$0077	;LSB - of total count

cpi		r16,1		;IS it 1 DIT? ... if so turn off POWER
brne	DBPushA	
rjmp	PowerOff	;off power

DBPushA:

cpi		r16,2		;IS it 2 DITS? .... if so, you wish to use last programmed time
brne	DBPushB
rjmp	DBPush2a


DBPushB:

cpi		r16,3		;IS it 3 DITS? .... if so, you wish to use EEPROM, Favorite time
brne	DBPushC
rjmp	UseEEPROMa


DBPushC:

rjmp	PowerOff	;Off power


UseEEPROMa:

ldi		r16,3
rcall	VariableBeep	;beep sonalert 3 times

rcall	UseEEPROM	;Get stored data in EEPROM for time and put into r30,31, r26,27
rjmp	Main2b		;use EEPROM data for timing


DBpush2a:			;Get last time interval

ldi		r16,2
rcall	VariableBeep	;beep sonalert two times

lds		r30,$61
lds		r31,$62		;low pair bytes

lds		r26,$63
lds		r27,$64		;high pair bytes

; RELOADED past information for delay cycle

rjmp	Main2b		;jump back to repeat the last timing cycle.


;----------------------------

UseEEPROM:			;read EEPROM data and put into r30,31, r26,27

ldi		r16,0		;location of EEPROM data
rcall	FetchEEPROM
mov		r30,r16		;low byte of low data pair (r30,31)

ldi		r16,1
rcall	FetchEEPROM
mov		r31,r16		;high byte of low data pair (r30,31)

ldi		r16,2
rcall	FetchEEPROM
mov		r26,r16		;low byte of high data pair (r26,27)

ldi		r16,3
rcall	FetchEEPROM
mov		r27,r16		;high byte of high data pair (r26,27)

ret


;----------------------------

;R28, R29 is input in minutes

BeepOutElapsed:		;This routine limited to 18.2 hours total elapsed time. Input is r28,29
					;   Time input is original MINUTES in r30,31 register pair
					;     put in r28,29 for processing
					;beeps out time in register pair r28,29 (y register)
					;3 short beeps for hours,  2 short beeps for 10 minute segments
					;1 short beep for one minute

;r8, r9 = scratch
;r10 = hours store
;r11 = 10 minute segment store

;SUBROUTINE

push	r28
push	r29

sbis	PIND,2		;skip if switch is up
rjmp	BOEend		;interruption occured, return early

clr		r10			;clear r10, hours count
clr		r11			;clear r11, 10 minute segment counts


BOE1:

mov		r8,r28
mov		r9,r29		;temporarily save values in r8,r9

sbiw	r28,60		;subtract 60 (1 hour) from word
brcs	BOE1a

inc		r10
rjmp	BOE1


BOE1a:

mov		r28,r8
mov		r29,r9


BOE2:

mov		r8,r28
mov		r9,r29

sbiw	r28,10		;subtract 10 minutes from word
brcs	BOE3

inc		r11
rjmp	BOE2


BOE3:

mov		r28,r8
mov		r29,r9

;	Beep Out elapsed time with 3 short beeps for hours, 2 short beeps for 10 min segments
;     and 1 short beep for minutes


BOE3a:

ldi		r16,4		;number of beeps per digit


BOE3aa:

tst		r10						;Beep out hours
breq	BOE4
rcall	Beep20thEscape
sbis	PIND,2					;skip if switch is up
rjmp	DecodeButtonPush2
dec		r10
brne	BOE3aa


BOE4:							;beep out 10 minute segments

rcall 	Delay500Escape			;create pause before next series
sbis	PIND,2					;skip if switch is up
rjmp	DecodeButtonPush2



BOE4a:

ldi		r16,2			;number of beeps per digit


BOE4aa:

tst		r11						;Beep out 10 minute segments
breq	BOE5
rcall	Beep20thEscape
sbis	PIND,2					;skip if switch is up
rjmp	DecodeButtonPush2

dec		r11
brne	BOE4aa


BOE5:

rcall 	Delay500Escape			;create pause before next series
sbis	PIND,2					;skip if switch is up
rjmp	DecodeButtonPush2



BOE5a:

ldi		r16,1			;number of beeps per digit


BOE5aa:

tst		r28						;Beep out minutes
breq	BOEend

rcall	Beep20thEscape
sbis	PIND,2					;skip if switch is up
rjmp	DecodeButtonPush2
dec		r28
brne	BOE5aa


BOEend:

pop		r29
pop		r28

ret


;--------------------------------

Beep20thEscape:				;value in r16 determines number of beeps

push	r16

Beep20th1:

tst		r16
breq	Beep20th2

sbi		PortB,0
rcall	Delay40thEscape
rcall	Delay40thEscape
cbi		PortB,0
rcall	Delay40thEscape
rcall	Delay40thEscape

dec		r16
brne	Beep20th1

rcall	Delay500Escape


Beep20th2:


pop		r16
ret


;--------------------------------

						;Enter countdown minutes into r30,31,
						;  Output goes to r30,31 (low pair) and r26,27 (high pair)
						; --- up to 10.2 days (r17 = 255)

CalculateTotalSeconds:


clr		r26
clr		r27				;high bits counting pair
clr		r30
clr		r31				;low bits counting pair

lds		r28,$0074	;No of dit counts (1 minute)
lds		r29,$0075	;No of dah counts (10 minutes)
lds		r17,$0076	;No of hold down counts (one hour)

rcall	Add3600seconds	;Add hour (r17)seconds to counter
rcall	Add600seconds	;Add 10 minute (r29) segment seconds to counter
rcall	Add60seconds	;Add minutes (r28) (60 seconds) for value in r17


;STORE away resultand values in r30,31 (low bytes), and r26,27 (high bytes)
;  for use IF repeat of a timing event is selected later

ret


;----------------------------

Main2:					;START of timing cycle

sbi		PortB,0			;ON Sonalert
rcall	Delay40th
cbi		PortB,0			;OFF Sonalert to indicate start of timing				

rcall	CalculateTotalSeconds	;R30,31 has minutes time entered, Output goes into
								; into r30,31, r26,27
								; --- up to 10.2 days 

sts		$61,r30
sts		$62,r31			;low byte register pair
sts		$63,r26
sts		$64,r27			;high byte register pair


Main2b:					;(Also a start point for repeat of timing cycle)

ldi		r16,1			;put 1 second into seconds counter $006B
sts		$006B,r16


;TOTAL seconds now entered into r30,31 (low) and r26,27 (high) pairs

cbr		r19,1			;Clear r19,0(1), alarm flag

sei						;ON Interrupts


;---------------------------------

Main3:					;One push of button reads out time left until alarm 
						;Two pushes (or more) shut off timer

sbis	PIND,2			;skip if switch is up
rcall	BeepOutOrStop

sbrs	r19,0			;skip if r19,0(1) is set (alarm flag on)
rjmp	Main3

ldi		r30,1
clr		r31				;put one into r30,31 pair

;SOUND ALARM - time is up

sbi		PortB,0				;ON sonalert
rcall	Delay1sec
cbi		PortB,0				;OFF sonalert (one second uninterruptible beep of Sonalert)
rcall	Delay40th
ldi		r16,1
rcall	BeepOutR16Escape	;give alarm beeps - time has expired - sound alarm
rcall	BeepOut80thEscape
rcall	BeepOutR16Escape	
rcall	BeepOut80thEscape

clr		r28
clr		r29				;zero r28,29 pair

sbis	PIND,2			;skip if switch is up
rjmp	DecodeButtonPush2

cbr		r19,128			;clear r19,7(128) - one second flag


Main4:					;This loop is AFTER ALARM goes off

sbrc	r19,7			;skip if r19,7(128) is clear - one second flag
rjmp	Main4aa

sbis	PIND,2
rjmp	DecodeButtonPush2
rjmp	Main4


Main4aa:

cbr		r19,128			;clear r19,7(128) - one second flag

lds		r16,$006B		;get current second
cpi		r16,20			;
breq	Main5

cpi		r16,40
breq	Main5

cpi		r16,00
breq	Main5a

rjmp	Main4


Main5:

rcall	BeepOutElapsed	;Beeps out elapsed minutes

rjmp	Main4


Main5a:				;Increment auto power off and turn off if just before two hours

adiw	r28,1		;increment minutes in r28,29 pair which is now acting as up counter for minutes
cpi		r29,28
brsh	Main5b		;OFF power - shut down just before two hours of beeping
rjmp	Main5


Main5b:

rjmp	PowerOff	;power off


;------------------------------

;Subroutine

Add60seconds:			;Adds 60 seconds for number of DIT counts in r28 -- to register pairs r30,31 - r26,27
						;  r28 contains dit count - add 60 seconds

tst		r28
breq	A60End


A60a:

adiw	r30,60
brcc	A60b

adiw	r26,1			;carry was set so add one to high r26,27 set
brcs	A60End


A60b:

dec		r28
rjmp	Add60seconds	;go another round?


A60End:

ret


;------------------------

Add600seconds:			;Adds TEN minutes to register pair r30,31 - r26,27
						;  Number of 10-sec segments in r29 - dah counts

tst		r29
breq	A600End

ldi		r16,10


A600a:

adiw	r30,60			;Add 60 second to r30,31 pair
brcc	A600b

adiw	r26,1			;Add one to high byte pair, r26,27
brcs	A600End


A600b:

dec		r16
brne	A600a

dec		r29
rjmp	Add600seconds


A600End:

ret


;-----------------------------

Add3600seconds:			;Adds 1 hour to r30,31 - r26,27 register pairs
						;  Hour, hold down, count is in r17

tst		r17
breq	A3600End

ldi		r16,60			;60 iterations for each r17 value


A3600a:

adiw	r30,60			;add 60 seconds to word
brcc	A3600b

adiw	r26,1			;add one to r26,27 high pair
brcs	A3600End


A3600b:

dec		r16
brne	A3600a

dec		r17
rjmp	Add3600seconds


A3600End:

ret

	
;-----------------------------

BeepOutR16Escape:				;Beeps out 40th second series of beeps, one second long for number of times in r16

push	r16
push	r17


BOR16Esca:

tst		r16
breq	BOR16EscEnd

ldi		r17,20			;cycle 20 40th second beeps


BOR16Escc:

sbi		PortB,0
rcall	Delay40thEscape
cbi		PortB,0
rcall	Delay40thEscape
dec		r17
brne	BOR16Escc

rcall	Delay250Escape	;quarter second pause between beep series

dec		r16
rjmp	BOR16Esca


BOR16EscEnd:

pop		r17
pop		r16

ret


;-----------------------------

BeepOut80th:			;give out one-half second beeps for number in r16

push	r16
push	r17


BO80tha:				;give out one-half second beeps for number in r16

tst		r16
breq	BO80thEnd

ldi		r17,20			;cycle 20 40th second beeps


BO80thc:

sbi		PortB,0
rcall	Delay80thEscape
cbi		PortB,0
rcall	Delay80thEscape
dec		r17
brne	BO80thc

rcall	Delay250Escape		;quarter second pause between beep series

dec		r16
rjmp	BO80tha


BO80thEnd:

pop		r17
pop		r16

ret


;-----------------------------

BeepOut80thEscape:				;Beeps out 80th second series of beeps, one-half second long for number of times in r16

push	r16
push	r17


BOR80thEsca:

tst		r16
breq	BOR80thEscEnd

ldi		r17,20			;cycle 20 40th second beeps


BOR80thEscc:

sbi		PortB,0
rcall	Delay80thEscape
cbi		PortB,0
rcall	Delay80thEscape
dec		r17
brne	BOR80thEscc

rcall	Delay250Escape	;quarter second pause between beep series

dec		r16
rjmp	BOR80thEsca


BOR80thEscEnd:

pop		r17
pop		r16

ret


;-----------------------------


BeepOutr16Value:		;Beeps out number of cycles put in r16 - 80th second on/off cycles

push	r16


BOV1:

tst		r16
breq	BeepOutR16ValueEnd

sbi		PortB,0			;on sonalert
rcall	delay80th
cbi		PortB,0			;off sonalert
rcall	delay80th
dec		r16
brne	BOV1


BeepOutR16ValueEnd:

pop		r16

ret


;-----------------------------

PowerOff:			;Turns battery power off for circuit after stall on Button Push

cli					;Off Interrupts
cbi		PortB,0		;Off Sonalert
cbi		PortB,2		;Off sustain line
rcall	Delay1sec


PowerOff1a:

sbic	PIND,2		;skip in button is down (PIND,2)
rjmp	PowerOff1

rcall	Delay40th
rjmp	PowerOff1a


PowerOff1:			;UNCONDITIONAL power off point -- no stall on Button Push

cbi		PortB,2		;OFF sustain line
rcall	Delay40th
rjmp	PowerOff1


;-----------------------------

BeepOutOrStop:

rcall	ButtonPush

lds		r16,$0077	;LSB of Total Count
cpi		r16,1
breq	BOOS1

rjmp	PowerOff1			;unconditional power off


BOOS1:

cli					;off interrupts

push	r26
push	r27
push	r30
push	r31

rcall	ConvertSecondsToMinutes	;minute results now in r28,29

pop		r31
pop		r30
pop		r26
pop		r27

sei					;ON interrupts

rcall	BeepOutElapsed 	;r28,29 has data in minutes


BOOSend:

ret


BOOS2:

sbi		PortB,0		;ON sonalert
rcall	Delay500
cbi		PortB,0		;OFF sonalet (Long Beep)

rjmp	BOOSend


;--------------------------

ConvertSecondsToMinutes:		;converts seconds count in r30,31 (low), r26,27 (high) and
								;  puts results in r28,29 upon exit.
			
								;  Max time is 18.2 hours


clr		r28
clr		r29			;clear scratch registers
ldi		r16,1		;for later addition purposes


CST1:

sbiw	r30,60		;subtract 60 seconds from total
brcc	CST2

sbiw	r26,1		;subtract 1 from high byte pair r26,27
brcc	CST2

rjmp	CSTend


CST2:

add		r28,r16		;add one minute to r28
brcc	CST1		;go for another round

add		r29,r16		;add one to high byte in r29
brcc	CST2

clr		r28
clr		r29


CSTend:

sts		$80,r28
sts		$81,r29

ret


;-------------------------------------

ClockCntDwnInt: 		;interrupt to update clock counters

push	r16

in		r16,SREG	;save flags
sts		$0060,r16

sbr		r19,128		;set r19, bit 7(128) -- one second flag 
					;(set each second)

;Following decrements set time for alarm countdown

sbrc	r19,0			;skip if r19,0(1) is clear 
rjmp	CC1b			;  If NOT clear, then countdown is skipped

sbiw	r30,1			;decrement r30,31 pair
brcc	CC1b

sbiw	r26,1			;decrement high bits, r26,27, byte by one
brcc	CC1b

sbr		r19,1			;ZERO count is reached - Set Alarm Flag, r19,0(1)


CC1b:

lds		r16,$006B		;load seconds counter $006B into r16

cp		r16,r21			;compare present second number to grabbed second number in r21

breq	CCDI1aa



CC1bbb:

inc		r16			;step one count of SECONDS counter
sts		$006B,r16


CC1bb:

cpi		r16,60
breq	CCDI1


rjmp	CCDIExit		;return since no action required


CCDI1aa:  

sbis	PortB,5
rjmp	Set1

;cbi		PortB,5
rjmp	Set2


Set1:

;sbi		PortB,5


Set2:

sbr		r19,32			;set r19,5(32) grabbed bit

rjmp	CC1bbb



CCDI1:

clr		r16			;zero r16 -- SECONDS counter
sts		$006B,r16

sbr		r19,64		;set R19 bit 6(64) -- the NEW ONE MINUTE flag


CCDI1Cntr:

lds		r16,$0068		;load low minutes byte into R16

inc		r16
sts		$0068,r16

brne	CCDI1a		;jump out if not zero (no overflow)

lds		r16,$0069		;high byte minutes
inc		r16			;increment high byte minutes
sts		$0069,r16


CCDI1a:

lds		r16,$0069		;check high byte if it is 5 (1440=  5 A0)
cpi		r16,5

breq	CCDI1b

rjmp	CCDIalarmExit


CCDI1b:			

lds		r16,$0068
cpi		r16,160		;1440 minutes reached -- it's now midnight

breq	CCDI1c

rjmp	CCDIalarmExit


CCDI1c:			;zero high and low bytes - it is midnight

clr		r16
sts		$0068,r16		;store away new start at zero of 1440 count
sts		$0069,r16

lds		r16,$006A		;get day of week and increment
inc		r16
sts		$006A,r16

cpi		r16,8

breq	CCDI1d


CCDI1e:

rjmp	CCDIalarmExit


CCDI1d:

ldi		r16,1		;load with 1 (Sunday)
sts		$006A,r16
rjmp	CCDI1e		;to continue


CCDIalarmExit:


CCDIExit:		;Exit this interrupt routine

lds		r16,$0060		;SREG temporary store
out		SREG,r16	;restore SREG data

pop		r16

reti				;exit


;-----------------------------------------

Delay80th:			;1/80th second delay using 3.58 xtl /8 Freq. = 447,443 Hz =  2.234 useconds each clock cycle

push	r16
push	r17

ldi		r16,64
ldi		r17,8


D80a:

dec		r16
breq	D80b

rjmp	D80a


D80b:

dec		r17
brne	D80a

pop		r17
pop		r16
ret


;--------------------------------

Delay80thEscape:			;1/80th second delay using 3.58 xtl/8 Freq. with escape

push	r16
push	r17

ldi		r16,47
ldi		r17,5


D80Esc1:

sbis	PIND,2	
rjmp	D80EscEnd

dec		r16
brne	D80Esc1


D80Esc2:

dec		r17
brne	D80Esc1


D80EscEnd:

pop		r17
pop		r16
ret


;--------------------------------

Delay40thEscape:

sbis	PIND,2			;skip if button is up, PIND,2
ret

rcall	Delay80thEscape
rcall	Delay80thEscape

ret


;--------------------------------

Delay250:			;quarter second delay

push	r16
ldi		r16,10


Delay250a:

rcall	Delay40th
dec		r16
brne	Delay250a

pop		r16

ret


;---------------------------------

Delay500:			;half second delay

push	r16
ldi		r16,20


Delay500a:

rcall	Delay40th
dec		r16
brne	Delay500a

pop		r16

ret

;--------------------------------

Delay250Escape:

push	r16
ldi		r16,10

sbis	PIND,2			;skip if button is up, PIND,2
rjmp	D250Eend

D250EscE:

rcall 	Delay40thEscape
dec		r16

brne	D250EscE


D250Eend:

pop		r16

ret

;--------------------------------

Delay500Escape:

push	r16
ldi		r16,20

sbis	PIND,2			;skip if button is up, PIND,2
rjmp	D500End


D500Esca:

rcall	Delay40thEscape
dec		r16

brne	D500Esca


D500End:

pop		r16

ret


;------------------------------

Delay40th:			;
					;
rcall	Delay80th
rcall	Delay80th

ret


;-------------------------

Delay1sec:

push	r16
ldi		r16,80


Delay1sec1:

rcall	Delay80th	;24.80929839-3/sec
dec		r16
brne	Delay1sec1


pop		r16

ret


;------------

Delay2sec:		;two second delay

push	r16
ldi		r16,160

rjmp	Delay1sec1


;------------------

FetchEEPROM:		;grabs data in permanent saved EEPROM
					;r16 has address location - Then r16 returns data

FEE:

sbic	EECR,EEWE	;Wait for completion of previous write
rjmp	FEE

out		EEAR,r16	;set up address in address register
sbi		EECR,EERE	;start EEPROM read by writing EERE

in		r16,EEDR	;read data from data register

ret					;code data now in r16


;-------------------------------------------------

WriteEEPROM:		;write to EEPROM - r17 has memory location, r16 has data


WEE:

sbic	EECR,EEPE	;Wait until ready for write
rjmp	WEE

out		EEAR,r17	;set up address in address register
out		EEDR,r16		;put contents of r16 into EEPROM data location

sbi		EECR,EEMPE	;write logical one to EEMPE

sbi		EECR,EEPE	;start eeprom write by setting EEPE

ret


;--------------------------------------------------

VariableBeep:		;r16 contains number of beeps

rcall	Delay40th
rcall	Delay40th
sbi		PortB,0		;ON sonalert
rcall	Delay40th
cbi		PortB,0

dec		r16
brne	VariableBeep

ret


;---------------------------------------

BeepIntervalLoop:	;Gives beep determined by period entered in EEPROM $04,05 (18.2 hours interval)

sbi		PortB,0		;ON sonalert for start beep
rcall	Delay500
cbi		PortB,0		;OFF sonalert

clr		r16
sts		$6B,r16		;clear seconds counter
cbr		r19,128		;clear seconds flag
sei					;ON interrupts


BIL1:

ldi		r16,$04		;EEPROM address start
rcall	FetchEEPROM
mov		r30,r16		;move in low byte of low pair
ldi		r16,$05		;next EEPROM address
rcall	FetchEEPROM
mov		r31,r16		;move in high byte of low pair

ldi		r16,$06		;EEPROM address start
rcall	FetchEEPROM
mov		r26,r16		;move in low byte of high pari
ldi		r16,$07		;next EEPROM address
rcall	FetchEEPROM
mov		r27,r16		;move in high byte of high pair


BIL2:

sbrs	r19,7		;skip if r19,7(128) is set - one second flag
rjmp	BIL2a

cbr		r19,128		;clear r19,7(128) one second flag
sbiw	r30,1		;subtract 1 from r30,31 paid
brcs	BIL2b

rjmp	BIL2		;go back for another round


BIL2b:	

sbiw	r26,1		;subtract 1 from high byte pair
brcs	BIL2c		;Time is up - go to beep

rjmp	BIL2		;for another round


BIL2a:

sbis	PIND,2		;skip if switch is up
rjmp	PowerOff

rjmp	BIL2


BIL2c:

sbi		PortB,0		;ON sonalert for 1/2 second beep
rcall	Delay500escape
sbis	PIND,2		;skip if switch is up
rjmp	PowerOff
cbi		PortB,0

rjmp	BIL1		;go back and reload new delay


;----------------------------

ProgramBeepInterval:	;programs interval between beeps

ldi		r16,4
rcall	VariableBeep

rcall	ButtonStall				;Get entry in minutes
rcall	CalculateTotalSeconds	;convert minutes to seconds
								;r30,31 have low pair bytes, r26,27 have high pair bytes

PBIEntry:
				
ldi		r17,$04			;address in EEPROM for interval store
mov		r16,r30			;low data byte of low pair
rcall	WriteEEPROM

ldi		r17,$05			;address in EEPROM for interval store
mov		r16,r31			;high data byte of low pair
rcall	WriteEEPROM

ldi		r17,$06			;address in EEPROM for interval store
mov		r16,r26			;low data byte high pair
rcall	WriteEEPROM

ldi		r17,$07			;address in EEPROM for interval store
mov		r16,r27			;high data byte high pair
rcall	WriteEEPROM

ldi		r16,4
rcall	VariableBeep

ret


;----------------------------

ProgramBeepInterval2:		;Programs beep interval in seconds

ldi		r16,5				;give 5 beeps
rcall	VariableBeep

rcall	ButtonStall

lds		r30,$77
lds		r31,$78				;Get button entry

clr		r26
clr		r27					;clear high bytes register pair

rjmp	PBIentry


;-----------------------------

Metronome:					;This section takes in two button pushes to determine
							;  length, then repeats the timing cycle until a long
							;  hold down of the button is received.
							;Any other two pushes start timing cycle over again.

sbic	PIND,2		;skip if switch is down
rjmp	Metronome	;cycle back until switch is pushed

clr		r28
clr		r29
clr		r30
clr		r31			;zero register pairs Y, Z

sbi		PortB,0		;ON sonalert
rcall	Delay40th
cbi		PortB,0		;OFF sonalert


MET1:

adiw	r28,1			;Add one to r28,29 pair
brcc	MET2

adiw	r30,1			;Add one to r30,31 pair due to overflow
cpi		r30,1			;Is it hold down "exit" push?
brsh	PowerOffSub
rjmp	MET2


PowerOffSub:

ldi		r16,4
rcall	VariableBeep

rjmp	METend


MET2:

sbis	PIND,2			;skip if switch is up
rjmp	MET1


MET4:

adiw	r28,1			;add one to r28,29 pair
brcc	MET3

adiw	r30,1			;add one to r30,31 pair due to overflow
brcc	MET3

rjmp	METend


MET3:

sbic	PIND,2			;skip is switch is down
rjmp	MET4

sts		$85,r28
sts		$86,r29
sts		$87,r30
sts		$88,r31			;store away interval delay captured


MET3a:

sbis	PIND,2			;skip if switch is up
rjmp	MET3a			;STALL if switch still down


MET7b:

sbi		PortB,0			;ON sonalert
rcall	Delay40th		
cbi		PortB,0			;OFF sonalert


MET7:

sbiw	r28,1			;subtract one from r28,29 pair
brcc	MET8

sbiw	r30,1			;subtract one from r30,31 pair due to overflow
brcc	MET8
rjmp	MET6			;loop back


MET8:

sbic	PIND,2			;skip if switch is down
rjmp	MET7
rjmp	Metronome		;start over for entry of new timing sequence


MET6:

lds		r28,$85
lds		r29,$86
lds		r30,$87
lds		r31,$88			;reload stored time interval

rjmp	MET7b


METend:

ret


;--------------------------------




